<?php

namespace Drupal\bibleref\Formatter;

use Drupal\bibleref\Plugin\Field\FieldType\BibleReferenceInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Theme\ThemeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class BibleRefFormatter implements BibleRefFormatterInterface, ContainerInjectionInterface {

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('theme.manager')
    );
  }

  /**
   * BibleRefFormatter constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module handler.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme_manager
   *   Theme manager.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ThemeManagerInterface $theme_manager
  ) {
    $this->moduleHandler = $module_handler;
    $this->themeManager = $theme_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function format(BibleReferenceInterface $item) {
    $args = [
      'format' => ':book :chapter.:verse',
      'values' => $item->toArray(),
      'args' => [
        ':book' => $item->getBook()->label(),
        ':chapter' => $item->getChapter(),
        ':verse' => $this->formatVerse($item),
      ],
    ];
    $context = [
      'item' => $item,
    ];
    $this->alter('bibleref_format_data', $args, $context);
    return new TranslatableMarkup($args['format'], $args['args'], ['context' => 'bibleref']);
  }

  /**
   * {@inheritdoc}
   */
  public function formatVerse(BibleReferenceInterface $item) {
    $values = $item->toArray();
    $args = [
      'format' => ':verse_from',
      'values' => [
        ':verse_from' => $values[BibleReferenceInterface::PROPERTY_VERSE_FROM],
        ':verse_to' => $values[BibleReferenceInterface::PROPERTY_VERSE_TO],
        ':verse_additional' => $values[BibleReferenceInterface::PROPERTY_VERSE_ADDITIONAL],
      ],
    ];
    if (!empty($values[BibleReferenceInterface::PROPERTY_VERSE_TO])) {
      $args['format'] = ':verse_from - :verse_to';
    }
    if (!empty($values[BibleReferenceInterface::PROPERTY_VERSE_ADDITIONAL])) {
      $args['format'] .= '; :verse_additional';
    }

    $context = [
      'item' => $item,
    ];
    $this->alter('bibleref_format_verse', $args, $context);
    return new TranslatableMarkup($args['format'], $args['values'], ['context' => 'bibleref']);
  }

  /**
   * Invoke alter hooks.
   */
  protected function alter($hook, &$data, &$context) {
    $this->moduleHandler->alter($hook, $data, $context);
    $this->themeManager->alter($hook, $data, $context);
  }

}
